@@ -1,19 +1,18 @@
-"""
- mbed CMSIS-DAP debugger
- Copyright (c) 2015-2015 ARM Limited
-
- Licensed under the Apache License, Version 2.0 (the "License");
- you may not use this file except in compliance with the License.
- You may obtain a copy of the License at
-
-     http://www.apache.org/licenses/LICENSE-2.0
-
- Unless required by applicable law or agreed to in writing, software
- distributed under the License is distributed on an "AS IS" BASIS,
- WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
- See the License for the specific language governing permissions and
- limitations under the License.
-"""
+# pyOCD debugger
+# Copyright (c) 2015-2019 Arm Limited
+# SPDX-License-Identifier: Apache-2.0
+#
+# Licensed under the Apache License, Version 2.0 (the "License");
+# you may not use this file except in compliance with the License.
+# You may obtain a copy of the License at
+#
+#     http://www.apache.org/licenses/LICENSE-2.0
+#
+# Unless required by applicable law or agreed to in writing, software
+# distributed under the License is distributed on an "AS IS" BASIS,
+# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+# See the License for the specific language governing permissions and
+# limitations under the License.
 from __future__ import print_function
 
 # Notes about this script
@@ -54,6 +53,7 @@ from itertools import product
 import traceback
 import json
 import sys
+import os
 
 # We expect arm-none-eabi-gdb-py to only run Python 2.x. If it moves
 # to Python 3, we need to know about it, so print a warning.
@@ -84,8 +84,12 @@ monitor_commands = [
     "set vector-catch a",
     "set step-into-interrupt on",
     "set step-into-interrupt off",
+    "threads status",
+    "readdp 0",
+    "status",
+    "show fault",
     # Invalid Command
-    "fawehfawoefhad"
+    "intentional_invalid_command_fawehfawoefhad"
 ]
 
 SIZE_TO_C_TYPE = {
@@ -100,6 +104,8 @@ TO_GDB_ACCESS = {
     "read_write": gdb.WP_ACCESS,
 }
 
+env_name = os.environ.get('TOX_ENV_NAME', '')
+env_file_name = ("_" + env_name) if env_name else ''
 
 def gdb_execute(cmd):
     print("Executing command:", cmd)
@@ -178,7 +184,7 @@ def valid_watchpoint(bkpt_size, bkpt_access, bkpt_addr):
 # Initial setup
 testn = int(gdb.parse_and_eval("$testn"))
 
-test_param_filename = "test_params%d.txt" % testn
+test_param_filename = "gdb_test_params%s_%d.txt" % (env_file_name, testn)
 with open(test_param_filename, "rb") as f:
     test_params = json.loads(f.read())
 
@@ -209,7 +215,7 @@ def run_test():
         gdb_execute("set mem inaccessible-by-default off")
 
         # Set raw logging
-        gdb_execute("set remotelogfile gdb_test_raw%d.txt" % testn)
+        gdb_execute("set remotelogfile gdb_test_raw%s_%d.txt" % (env_file_name, testn))
 
         # Connect to server
         gdb_execute("target remote localhost:%d" % test_port)
@@ -263,6 +269,7 @@ def run_test():
         event = yield(DEFAULT_TIMEOUT)
         if not is_event_breakpoint(event, breakpoint):
             fail_count += 1
+            print("Expected: %s\nReceived: %s" % (breakpoint, event))
             print("Error - could not set pc to function")
         breakpoint.delete()
 
@@ -467,7 +474,7 @@ def run_test():
         fail_count += 1
     finally:
         test_result["fail_count"] = fail_count
-        test_result_filename = "test_results%d.txt" % testn
+        test_result_filename = "gdb_test_results%s_%d.txt" % (env_file_name, testn)
         with open(test_result_filename, "wb") as f:
             f.write(json.dumps(test_result))
         gdb_execute("detach")
